<?php

namespace Litecms\Coins\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Coins\Http\Requests\CoinRequest;
use Litecms\Coins\Interfaces\CoinRepositoryInterface;
use Litecms\Coins\Models\Coin;

/**
 * Resource controller class for coin.
 */
class CoinResourceController extends BaseController
{

    /**
     * Initialize coin resource controller.
     *
     * @param type CoinRepositoryInterface $coin
     *
     * @return null
     */
    public function __construct(CoinRepositoryInterface $coin)
    {
        parent::__construct();
        $this->repository = $coin;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Coins\Repositories\Criteria\CoinResourceCriteria::class);
    }

    /**
     * Display a list of coin.
     *
     * @return Response
     */
    public function index(CoinRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Coins\Repositories\Presenter\CoinPresenter::class)
                ->$function();
        }

        $coins = $this->repository->paginate();

        return $this->response->title(trans('coins::coin.names'))
            ->view('coins::coin.index', true)
            ->data(compact('coins'))
            ->output();
    }

    /**
     * Display coin.
     *
     * @param Request $request
     * @param Model   $coin
     *
     * @return Response
     */
    public function show(CoinRequest $request, Coin $coin)
    {

        if ($coin->exists) {
            $view = 'coins::coin.show';
        } else {
            $view = 'coins::coin.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('coins::coin.name'))
            ->data(compact('coin'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new coin.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CoinRequest $request)
    {

        $coin = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('coins::coin.name')) 
            ->view('coins::coin.create', true) 
            ->data(compact('coin'))
            ->output();
    }

    /**
     * Create new coin.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CoinRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $coin                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('coins::coin.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('coins/coin/' . $coin->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/coins/coin'))
                ->redirect();
        }

    }

    /**
     * Show coin for editing.
     *
     * @param Request $request
     * @param Model   $coin
     *
     * @return Response
     */
    public function edit(CoinRequest $request, Coin $coin)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('coins::coin.name'))
            ->view('coins::coin.edit', true)
            ->data(compact('coin'))
            ->output();
    }

    /**
     * Update the coin.
     *
     * @param Request $request
     * @param Model   $coin
     *
     * @return Response
     */
    public function update(CoinRequest $request, Coin $coin)
    {
        try {
            $attributes = $request->all();

            $coin->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('coins::coin.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('coins/coin/' . $coin->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('coins/coin/' . $coin->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the coin.
     *
     * @param Model   $coin
     *
     * @return Response
     */
    public function destroy(CoinRequest $request, Coin $coin)
    {
        try {

            $coin->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('coins::coin.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('coins/coin/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('coins/coin/' . $coin->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple coin.
     *
     * @param Model   $coin
     *
     * @return Response
     */
    public function delete(CoinRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('coins::coin.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('coins/coin'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/coins/coin'))
                ->redirect();
        }

    }

    /**
     * Restore deleted coins.
     *
     * @param Model   $coin
     *
     * @return Response
     */
    public function restore(CoinRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('coins::coin.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/coins/coin'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/coins/coin/'))
                ->redirect();
        }

    }

}
